"""
插件化配置架构
通过插件系统实现配置模块的完全解耦和热插拔
"""

import asyncio
import importlib
import inspect
import json
import logging
from abc import ABC, abstractmethod
from dataclasses import dataclass, field
from enum import Enum
from pathlib import Path
from typing import Dict, Any, List, Type, Optional, Set, Union

import yaml

logger = logging.getLogger(__name__)


class PluginType(Enum):
    """插件类型"""
    CONFIG_PROVIDER = "config_provider"
    CONFIG_VALIDATOR = "config_validator"
    CONFIG_PERSISTENCE = "config_persistence"
    CONFIG_EXPORTER = "config_exporter"
    CONFIG_WATCHER = "config_watcher"
    CONFIG_TRANSFORMER = "config_transformer"


@dataclass
class PluginMetadata:
    """插件元数据"""
    name: str
    version: str
    description: str
    author: str
    plugin_type: PluginType
    dependencies: List[str] = field(default_factory=list)
    config_schema: Optional[Dict[str, Any]] = None
    enabled: bool = True


@dataclass
class PluginLoadResult:
    """插件加载结果"""
    success: bool
    plugin_metadata: Optional[PluginMetadata] = None
    error_message: Optional[str] = None
    plugin_instance: Optional[Any] = None


class IPlugin(ABC):
    """插件接口"""

    @property
    @abstractmethod
    def metadata(self) -> PluginMetadata:
        """插件元数据"""
        pass

    @abstractmethod
    async def initialize(self, config: Dict[str, Any]) -> bool:
        """初始化插件"""
        pass

    @abstractmethod
    async def start(self) -> bool:
        """启动插件"""
        pass

    @abstractmethod
    async def stop(self) -> bool:
        """停止插件"""
        pass

    @abstractmethod
    async def cleanup(self) -> bool:
        """清理插件资源"""
        pass


class IConfigProviderPlugin(IPlugin):
    """配置提供者插件接口"""

    @abstractmethod
    async def get_config(self, key: str, default: Any = None) -> Any:
        pass

    @abstractmethod
    async def set_config(self, key: str, value: Any, persist: bool = True) -> bool:
        pass

    @abstractmethod
    async def list_configs(self, pattern: str = "*") -> Dict[str, Any]:
        pass

    @abstractmethod
    async def reload(self) -> bool:
        pass


class IConfigValidatorPlugin(IPlugin):
    """配置验证器插件接口"""

    @abstractmethod
    async def validate_config(self, key: str, value: Any) -> tuple[bool, str]:
        pass

    @abstractmethod
    def get_validation_schema(self) -> Dict[str, Any]:
        pass

    @abstractmethod
    def get_supported_keys(self) -> Set[str]:
        pass


class IConfigPersistencePlugin(IPlugin):
    """配置持久化插件接口"""

    @abstractmethod
    async def load_config(self) -> Dict[str, Any]:
        pass

    @abstractmethod
    async def save_config(self, config: Dict[str, Any]) -> bool:
        pass

    @abstractmethod
    async def backup_config(self, backup_name: str) -> bool:
        pass

    @abstractmethod
    async def restore_config(self, backup_name: str) -> bool:
        pass


class PluginRegistry:
    """插件注册表"""

    def __init__(self):
        self._plugins: Dict[str, IPlugin] = {}
        self._plugins_by_type: Dict[PluginType, List[IPlugin]] = {}
        self._plugin_configs: Dict[str, Dict[str, Any]] = {}
        self._lock = asyncio.Lock()

    async def register_plugin(self, plugin: IPlugin, config: Dict[str, Any] = None) -> PluginLoadResult:
        """注册插件"""
        async with self._lock:
            metadata = plugin.metadata

            # 检查依赖
            if not await self._check_dependencies(metadata.dependencies):
                return PluginLoadResult(
                    success=False,
                    error_message=f"Dependencies not satisfied: {metadata.dependencies}"
                )

            try:
                # 初始化插件
                if await plugin.initialize(config or {}):
                    # 启动插件
                    if await plugin.start():
                        self._plugins[metadata.name] = plugin
                        self._plugin_configs[metadata.name] = config or {}

                        # 按类型分类
                        if metadata.plugin_type not in self._plugins_by_type:
                            self._plugins_by_type[metadata.plugin_type] = []
                        self._plugins_by_type[metadata.plugin_type].append(plugin)

                        logger.info(f"Plugin {metadata.name} registered successfully")
                        return PluginLoadResult(
                            success=True,
                            plugin_metadata=metadata,
                            plugin_instance=plugin
                        )
                    else:
                        await plugin.cleanup()
                        return PluginLoadResult(
                            success=False,
                            error_message="Plugin failed to start"
                        )
                else:
                    return PluginLoadResult(
                        success=False,
                        error_message="Plugin failed to initialize"
                    )

            except Exception as e:
                logger.error(f"Failed to register plugin {metadata.name}: {e}")
                return PluginLoadResult(
                    success=False,
                    error_message=str(e)
                )

    async def unregister_plugin(self, plugin_name: str) -> bool:
        """注销插件"""
        async with self._lock:
            if plugin_name not in self._plugins:
                return False

            plugin = self._plugins[plugin_name]
            metadata = plugin.metadata

            try:
                await plugin.stop()
                await plugin.cleanup()

                # 从注册表移除
                self._plugins.pop(plugin_name, None)
                self._plugin_configs.pop(plugin_name, None)

                # 从类型分类中移除
                if metadata.plugin_type in self._plugins_by_type:
                    self._plugins_by_type[metadata.plugin_type] = [
                        p for p in self._plugins_by_type[metadata.plugin_type]
                        if p.metadata.name != plugin_name
                    ]

                logger.info(f"Plugin {plugin_name} unregistered successfully")
                return True

            except Exception as e:
                logger.error(f"Failed to unregister plugin {plugin_name}: {e}")
                return False

    def get_plugin(self, plugin_name: str) -> Optional[IPlugin]:
        """获取插件"""
        return self._plugins.get(plugin_name)

    def get_plugins_by_type(self, plugin_type: PluginType) -> List[IPlugin]:
        """按类型获取插件"""
        return self._plugins_by_type.get(plugin_type, [])

    def list_plugins(self) -> List[PluginMetadata]:
        """列出所有插件"""
        return [plugin.metadata for plugin in self._plugins.values()]

    async def _check_dependencies(self, dependencies: List[str]) -> bool:
        """检查插件依赖"""
        for dep in dependencies:
            if dep not in self._plugins:
                return False
        return True


class PluginLoader:
    """插件加载器"""

    def __init__(self, registry: PluginRegistry):
        self.registry = registry
        self._loaded_modules: Dict[str, Any] = {}

    async def load_plugin_from_file(self, file_path: Union[str, Path],
                                   config: Dict[str, Any] = None) -> PluginLoadResult:
        """从文件加载插件"""
        file_path = Path(file_path)

        if not file_path.exists():
            return PluginLoadResult(
                success=False,
                error_message=f"Plugin file not found: {file_path}"
            )

        try:
            # 动态导入模块
            spec = importlib.util.spec_from_file_location(file_path.stem, file_path)
            module = importlib.util.module_from_spec(spec)
            spec.loader.exec_module(module)
            self._loaded_modules[file_path.stem] = module

            # 查找插件类
            plugin_class = self._find_plugin_class(module)
            if not plugin_class:
                return PluginLoadResult(
                    success=False,
                    error_message="No plugin class found in module"
                )

            # 创建插件实例
            plugin_instance = plugin_class()

            # 验证插件接口
            if not isinstance(plugin_instance, IPlugin):
                return PluginLoadResult(
                    success=False,
                    error_message="Plugin class does not implement IPlugin interface"
                )

            # 注册插件
            return await self.registry.register_plugin(plugin_instance, config)

        except Exception as e:
            return PluginLoadResult(
                success=False,
                error_message=f"Failed to load plugin: {str(e)}"
            )

    async def load_plugin_from_module(self, module_name: str,
                                     config: Dict[str, Any] = None) -> PluginLoadResult:
        """从模块名加载插件"""
        try:
            module = importlib.import_module(module_name)
            self._loaded_modules[module_name] = module

            plugin_class = self._find_plugin_class(module)
            if not plugin_class:
                return PluginLoadResult(
                    success=False,
                    error_message="No plugin class found in module"
                )

            plugin_instance = plugin_class()
            if not isinstance(plugin_instance, IPlugin):
                return PluginLoadResult(
                    success=False,
                    error_message="Plugin class does not implement IPlugin interface"
                )

            return await self.registry.register_plugin(plugin_instance, config)

        except Exception as e:
            return PluginLoadResult(
                success=False,
                error_message=f"Failed to load plugin from module {module_name}: {str(e)}"
            )

    def _find_plugin_class(self, module) -> Optional[Type[IPlugin]]:
        """查找插件类"""
        for name, obj in inspect.getmembers(module, inspect.isclass):
            if (issubclass(obj, IPlugin) and
                obj != IPlugin and
                not inspect.isabstract(obj)):
                return obj
        return None


class PluginManager:
    """插件管理器"""

    def __init__(self, config_path: Optional[Union[str, Path]] = None):
        self.registry = PluginRegistry()
        self.loader = PluginLoader(self.registry)
        self.config_path = Path(config_path) if config_path else None
        self._config: Dict[str, Any] = {}

    async def initialize(self):
        """初始化插件管理器"""
        if self.config_path and self.config_path.exists():
            await self._load_config()

        # 加载默认插件
        await self._load_default_plugins()

        # 加载配置文件中定义的插件
        await self._load_configured_plugins()

        logger.info("Plugin manager initialized")

    async def _load_config(self):
        """加载配置文件"""
        try:
            with open(self.config_path, 'r', encoding='utf-8') as f:
                if self.config_path.suffix.lower() in ['.yaml', '.yml']:
                    self._config = yaml.safe_load(f)
                else:
                    self._config = json.load(f)
        except Exception as e:
            logger.error(f"Failed to load plugin config: {e}")
            self._config = {}

    async def _load_default_plugins(self):
        """加载默认插件"""
        # 内置插件
        default_plugins = [
            "mcpstore.core.configuration.builtin_plugins.file_config_provider",
            "mcpstore.core.configuration.builtin_plugins.env_config_provider",
            "mcpstore.core.configuration.builtin_plugins.json_config_validator",
        ]

        for plugin_module in default_plugins:
            try:
                result = await self.loader.load_plugin_from_module(plugin_module)
                if result.success:
                    logger.info(f"Default plugin {plugin_module} loaded")
                else:
                    logger.warning(f"Failed to load default plugin {plugin_module}: {result.error_message}")
            except Exception as e:
                logger.warning(f"Failed to load default plugin {plugin_module}: {e}")

    async def _load_configured_plugins(self):
        """加载配置文件中定义的插件"""
        plugins_config = self._config.get("plugins", {})
        for plugin_name, plugin_config in plugins_config.items():
            if not plugin_config.get("enabled", True):
                continue

            plugin_path = plugin_config.get("path")
            plugin_module = plugin_config.get("module")

            if plugin_path:
                result = await self.loader.load_plugin_from_file(plugin_path, plugin_config)
            elif plugin_module:
                result = await self.loader.load_plugin_from_module(plugin_module, plugin_config)
            else:
                logger.warning(f"Plugin {plugin_name} has no path or module specified")
                continue

            if result.success:
                logger.info(f"Configured plugin {plugin_name} loaded")
            else:
                logger.error(f"Failed to load configured plugin {plugin_name}: {result.error_message}")

    async def reload_plugin(self, plugin_name: str) -> bool:
        """重新加载插件"""
        plugin = self.registry.get_plugin(plugin_name)
        if not plugin:
            return False

        config = self.registry._plugin_configs.get(plugin_name, {})
        metadata = plugin.metadata

        # 先注销
        await self.registry.unregister_plugin(plugin_name)

        # 重新加载
        if plugin_name in self.registry._loaded_modules:
            # 重新导入模块
            module = self.registry._loaded_modules[plugin_name]
            importlib.reload(module)

        # 根据元数据重新加载
        # 这里需要根据插件的来源重新加载

        return True

    async def shutdown(self):
        """关闭插件管理器"""
        # 注销所有插件
        plugins = list(self.registry._plugins.keys())
        for plugin_name in plugins:
            await self.registry.unregister_plugin(plugin_name)

        logger.info("Plugin manager shutdown")


# 插件化配置管理器
class PluginBasedConfigManager:
    """基于插件的配置管理器"""

    def __init__(self, plugin_manager: Optional[PluginManager] = None):
        self.plugin_manager = plugin_manager or PluginManager()
        self._config_cache: Dict[str, Any] = {}
        self._cache_lock = asyncio.Lock()
        self._provider_priority: List[str] = []

    async def initialize(self):
        """初始化配置管理器"""
        await self.plugin_manager.initialize()

        # 确定配置提供者优先级
        await self._setup_provider_priority()

        logger.info("Plugin-based config manager initialized")

    async def _setup_provider_priority(self):
        """设置配置提供者优先级"""
        providers = self.plugin_manager.registry.get_plugins_by_type(PluginType.CONFIG_PROVIDER)

        # 按优先级排序（环境变量 > 文件 > 远程等）
        priority_order = ["env", "file", "remote", "database"]
        sorted_providers = []

        for provider_type in priority_order:
            matching = [p for p in providers if provider_type in p.metadata.name.lower()]
            sorted_providers.extend(matching)

        # 添加其他提供者
        others = [p for p in providers if p not in sorted_providers]
        sorted_providers.extend(others)

        self._provider_priority = [p.metadata.name for p in sorted_providers]

    async def get_config(self, key: str, default: Any = None) -> Any:
        """获取配置值（按提供者优先级）"""
        async with self._cache_lock:
            if key in self._config_cache:
                return self._config_cache[key]

            # 按优先级从提供者获取配置
            for provider_name in self._provider_priority:
                provider = self.plugin_manager.registry.get_plugin(provider_name)
                if isinstance(provider, IConfigProviderPlugin):
                    try:
                        value = await provider.get_config(key, None)
                        if value is not None:
                            self._config_cache[key] = value
                            return value
                    except Exception as e:
                        logger.warning(f"Provider {provider_name} failed to get config {key}: {e}")

            # 返回默认值
            if default is not None:
                self._config_cache[key] = default
            return default

    async def set_config(self, key: str, value: Any, persist: bool = True) -> bool:
        """设置配置值"""
        async with self._cache_lock:
            # 更新缓存
            old_value = self._config_cache.get(key)
            self._config_cache[key] = value

            # 通知所有提供者
            providers = self.plugin_manager.registry.get_plugins_by_type(PluginType.CONFIG_PROVIDER)
            success_count = 0

            for provider in providers:
                if isinstance(provider, IConfigProviderPlugin):
                    try:
                        if await provider.set_config(key, value, persist):
                            success_count += 1
                    except Exception as e:
                        logger.warning(f"Provider {provider.metadata.name} failed to set config {key}: {e}")

            return success_count > 0

    async def validate_config(self, key: str, value: Any) -> tuple[bool, str]:
        """验证配置值"""
        validators = self.plugin_manager.registry.get_plugins_by_type(PluginType.CONFIG_VALIDATOR)

        for validator in validators:
            if isinstance(validator, IConfigValidatorPlugin):
                try:
                    is_valid, message = await validator.validate_config(key, value)
                    if not is_valid:
                        return False, message
                except Exception as e:
                    logger.warning(f"Validator {validator.metadata.name} failed: {e}")

        return True, ""

    async def reload_config(self) -> bool:
        """重新加载配置"""
        providers = self.plugin_manager.registry.get_plugins_by_type(PluginType.CONFIG_PROVIDER)

        for provider in providers:
            if isinstance(provider, IConfigProviderPlugin):
                try:
                    await provider.reload()
                except Exception as e:
                    logger.warning(f"Provider {provider.metadata.name} failed to reload: {e}")

        # 清空缓存
        async with self._cache_lock:
            self._config_cache.clear()

        return True


# 示例插件实现
class FileConfigProviderPlugin(IConfigProviderPlugin):
    """文件配置提供者插件示例"""

    def __init__(self, config_file: str = "config.toml"):
        self.config_file = Path(config_file)
        self._config: Dict[str, Any] = {}
        self._metadata = PluginMetadata(
            name="file_config_provider",
            version="1.0.0",
            description="File-based configuration provider",
            author="MCPStore",
            plugin_type=PluginType.CONFIG_PROVIDER
        )

    @property
    def metadata(self) -> PluginMetadata:
        return self._metadata

    async def initialize(self, config: Dict[str, Any]) -> bool:
        try:
            if self.config_file.exists():
                # 根据文件扩展名选择解析器
                if self.config_file.suffix.lower() == '.json':
                    with open(self.config_file, 'r') as f:
                        self._config = json.load(f)
                elif self.config_file.suffix.lower() in ['.toml']:
                    import toml
                    with open(self.config_file, 'r') as f:
                        self._config = toml.load(f)
            return True
        except Exception as e:
            logger.error(f"Failed to initialize file config provider: {e}")
            return False

    async def start(self) -> bool:
        return True

    async def stop(self) -> bool:
        return True

    async def cleanup(self) -> bool:
        return True

    async def get_config(self, key: str, default: Any = None) -> Any:
        keys = key.split('.')
        value = self._config
        try:
            for k in keys:
                value = value[k]
            return value
        except (KeyError, TypeError):
            return default

    async def set_config(self, key: str, value: Any, persist: bool = True) -> bool:
        keys = key.split('.')
        config = self._config
        for k in keys[:-1]:
            config = config.setdefault(k, {})
        config[keys[-1]] = value

        if persist:
            return await self._save_config()
        return True

    async def list_configs(self, pattern: str = "*") -> Dict[str, Any]:
        return self._config

    async def reload(self) -> bool:
        return await self.initialize({})

    async def _save_config(self) -> bool:
        try:
            with open(self.config_file, 'w') as f:
                if self.config_file.suffix.lower() == '.json':
                    json.dump(self._config, f, indent=2)
                elif self.config_file.suffix.lower() in ['.toml']:
                    import toml
                    toml.dump(self._config, f)
            return True
        except Exception as e:
            logger.error(f"Failed to save config: {e}")
            return False


# 使用示例
async def setup_plugin_based_config():
    """设置基于插件的配置系统"""
    plugin_manager = PluginManager("plugins_config.json")
    config_manager = PluginBasedConfigManager(plugin_manager)

    await config_manager.initialize()

    # 使用配置管理器
    value = await config_manager.get_config("server.port", 8080)
    print(f"Server port: {value}")

    await config_manager.set_config("server.host", "0.0.0.0")

    return config_manager


"""
事件驱动配置架构
通过事件系统解耦配置模块之间的直接依赖
"""

import asyncio
import logging
from abc import ABC, abstractmethod
from dataclasses import dataclass, field
from datetime import datetime
from enum import Enum
from typing import Dict, Any, List, Optional

logger = logging.getLogger(__name__)


class ConfigEventType(Enum):
    """配置事件类型"""
    CONFIG_CHANGED = "config_changed"
    CONFIG_LOADED = "config_loaded"
    CONFIG_SAVED = "config_saved"
    CONFIG_VALIDATED = "config_validated"
    CONFIG_ERROR = "config_error"
    SNAPSHOT_CREATED = "snapshot_created"
    SERVICE_REGISTERED = "service_registered"


@dataclass
class ConfigEvent:
    """配置事件"""
    event_type: ConfigEventType
    key: Optional[str] = None
    old_value: Any = None
    new_value: Any = None
    source: Optional[str] = None
    timestamp: datetime = field(default_factory=datetime.now)
    metadata: Dict[str, Any] = field(default_factory=dict)
    correlation_id: Optional[str] = None


class IEventHandler(ABC):
    """事件处理器接口"""

    @abstractmethod
    async def handle(self, event: ConfigEvent) -> None:
        """处理事件"""
        pass

    @property
    @abstractmethod
    def handled_events(self) -> List[ConfigEventType]:
        """处理的事件类型"""
        pass


class EventBus:
    """配置事件总线"""

    def __init__(self):
        self._handlers: Dict[ConfigEventType, List[IEventHandler]] = {}
        self._global_handlers: List[IEventHandler] = []
        self._lock = asyncio.Lock()
        self._event_history: List[ConfigEvent] = []
        self._max_history = 1000

    async def subscribe(self, event_type: ConfigEventType, handler: IEventHandler):
        """订阅事件"""
        async with self._lock:
            if event_type not in self._handlers:
                self._handlers[event_type] = []
            self._handlers[event_type].append(handler)
            logger.debug(f"Handler subscribed to {event_type}")

    async def subscribe_all(self, handler: IEventHandler):
        """订阅所有事件"""
        async with self._lock:
            self._global_handlers.append(handler)
            logger.debug("Global handler subscribed")

    async def unsubscribe(self, event_type: ConfigEventType, handler: IEventHandler):
        """取消订阅"""
        async with self._lock:
            if event_type in self._handlers:
                try:
                    self._handlers[event_type].remove(handler)
                    logger.debug(f"Handler unsubscribed from {event_type}")
                except ValueError:
                    pass

    async def publish(self, event: ConfigEvent) -> None:
        """发布事件"""
        async with self._lock:
            # 记录事件历史
            self._event_history.append(event)
            if len(self._event_history) > self._max_history:
                self._event_history.pop(0)

        # 分发事件
        await self._dispatch_event(event)

    async def _dispatch_event(self, event: ConfigEvent) -> None:
        """分发事件到处理器"""
        handlers = []

        # 获取特定事件类型的处理器
        if event.event_type in self._handlers:
            handlers.extend(self._handlers[event.event_type])

        # 添加全局处理器
        handlers.extend(self._global_handlers)

        # 并发处理事件
        tasks = []
        for handler in handlers:
            if not handler.handled_events or event.event_type in handler.handled_events:
                task = asyncio.create_task(self._safe_handle(handler, event))
                tasks.append(task)

        if tasks:
            await asyncio.gather(*tasks, return_exceptions=True)

    async def _safe_handle(self, handler: IEventHandler, event: ConfigEvent):
        """安全处理事件"""
        try:
            await handler.handle(event)
        except Exception as e:
            logger.error(f"Event handler error: {e}", exc_info=True)

    def get_event_history(self, event_type: Optional[ConfigEventType] = None,
                         since: Optional[datetime] = None) -> List[ConfigEvent]:
        """获取事件历史"""
        history = self._event_history

        if event_type:
            history = [e for e in history if e.event_type == event_type]

        if since:
            history = [e for e in history if e.timestamp >= since]

        return history


# 全局事件总线
_global_event_bus: Optional[EventBus] = None


def get_event_bus() -> EventBus:
    """获取全局事件总线"""
    global _global_event_bus
    if _global_event_bus is None:
        _global_event_bus = EventBus()
    return _global_event_bus


class EventDrivenConfigProvider:
    """事件驱动的配置提供者"""

    def __init__(self, event_bus: Optional[EventBus] = None):
        self.event_bus = event_bus or get_event_bus()
        self._config: Dict[str, Any] = {}
        self._sources: Dict[str, str] = {}
        self._lock = asyncio.Lock()

    async def get_config(self, key: str, default: Any = None) -> Any:
        """获取配置"""
        async with self._lock:
            value = self._config.get(key, default)
            source = self._sources.get(key, "unknown")

            # 发布配置访问事件（可选）
            # await self.event_bus.publish(ConfigEvent(
            #     event_type=ConfigEventType.CONFIG_ACCESSED,
            #     key=key,
            #     new_value=value,
            #     source=source
            # ))

            return value

    async def set_config(self, key: str, value: Any, persist: bool = True,
                        source: str = "runtime") -> bool:
        """设置配置"""
        async with self._lock:
            old_value = self._config.get(key)
            old_source = self._sources.get(key)

            # 设置值
            self._config[key] = value
            self._sources[key] = source

            # 发布变更事件
            await self.event_bus.publish(ConfigEvent(
                event_type=ConfigEventType.CONFIG_CHANGED,
                key=key,
                old_value=old_value,
                new_value=value,
                source=source,
                metadata={"persist": persist}
            ))

            return True

    async def load_config_from_dict(self, config: Dict[str, Any],
                                   source: str = "loaded") -> None:
        """从字典加载配置"""
        async with self._lock:
            for key, value in config.items():
                old_value = self._config.get(key)
                self._config[key] = value
                self._sources[key] = source

            await self.event_bus.publish(ConfigEvent(
                event_type=ConfigEventType.CONFIG_LOADED,
                source=source,
                metadata={"keys_loaded": list(config.keys())}
            ))


# 配置事件处理器
class ConfigValidationHandler(IEventHandler):
    """配置验证处理器"""

    def __init__(self, validator=None):
        self.validator = validator

    @property
    def handled_events(self) -> List[ConfigEventType]:
        return [ConfigEventType.CONFIG_CHANGED]

    async def handle(self, event: ConfigEvent) -> None:
        """处理配置变更验证"""
        if self.validator and event.key:
            is_valid, message = await self.validator.validate_config(event.key, event.new_value)
            if not is_valid:
                await get_event_bus().publish(ConfigEvent(
                    event_type=ConfigEventType.CONFIG_ERROR,
                    key=event.key,
                    new_value=event.new_value,
                    metadata={"validation_error": message, "correlation_id": event.correlation_id}
                ))
            else:
                await get_event_bus().publish(ConfigEvent(
                    event_type=ConfigEventType.CONFIG_VALIDATED,
                    key=event.key,
                    new_value=event.new_value,
                    correlation_id=event.correlation_id
                ))


class ConfigPersistenceHandler(IEventHandler):
    """配置持久化处理器"""

    def __init__(self, persistence_service=None):
        self.persistence_service = persistence_service

    @property
    def handled_events(self) -> List[ConfigEventType]:
        return [ConfigEventType.CONFIG_CHANGED, ConfigEventType.CONFIG_SAVED]

    async def handle(self, event: ConfigEvent) -> None:
        """处理配置持久化"""
        if self.persistence_service and event.metadata.get("persist", True):
            if event.event_type == ConfigEventType.CONFIG_CHANGED:
                # 异步持久化
                asyncio.create_task(self._persist_config(event))
            elif event.event_type == ConfigEventType.CONFIG_SAVED:
                logger.info(f"Config saved to {event.source}")

    async def _persist_config(self, event: ConfigEvent):
        """持久化配置"""
        try:
            if event.key is not None:
                await self.persistence_service.save_config_key(event.key, event.new_value)
            else:
                # 批量保存
                await self.persistence_service.save_all_config()
        except Exception as e:
            logger.error(f"Failed to persist config: {e}")


class ConfigSnapshotHandler(IEventHandler):
    """配置快照处理器"""

    def __init__(self, snapshot_service=None):
        self.snapshot_service = snapshot_service

    @property
    def handled_events(self) -> List[ConfigEventType]:
        return [ConfigEventType.CONFIG_CHANGED, ConfigEventType.SNAPSHOT_CREATED]

    async def handle(self, event: ConfigEvent) -> None:
        """处理配置快照"""
        if self.snapshot_service and event.event_type == ConfigEventType.CONFIG_CHANGED:
            # 重要配置变更时创建快照
            if self._should_create_snapshot(event):
                await self.snapshot_service.create_snapshot(
                    reason=f"Config change: {event.key}",
                    trigger_event=event
                )

    def _should_create_snapshot(self, event: ConfigEvent) -> bool:
        """判断是否应该创建快照"""
        # 关键配置变更
        critical_keys = ["server.port", "database.url", "security.enabled"]
        return event.key in critical_keys


class ConfigChangeLogger(IEventHandler):
    """配置变更日志处理器"""

    @property
    def handled_events(self) -> List[ConfigEventType]:
        return [ConfigEventType.CONFIG_CHANGED, ConfigEventType.CONFIG_ERROR]

    async def handle(self, event: ConfigEvent) -> None:
        """记录配置变更日志"""
        if event.event_type == ConfigEventType.CONFIG_CHANGED:
            logger.info(f"Config changed: {event.key} = {event.new_value} (from {event.source})")
        elif event.event_type == ConfigEventType.CONFIG_ERROR:
            logger.error(f"Config error: {event.key} = {event.new_value} - {event.metadata.get('validation_error')}")


# 集成配置管理器
class EventDrivenConfigManager:
    """事件驱动配置管理器"""

    def __init__(self, event_bus: Optional[EventBus] = None):
        self.event_bus = event_bus or get_event_bus()
        self.provider = EventDrivenConfigProvider(self.event_bus)
        self._handlers: List[IEventHandler] = []

    async def initialize(self):
        """初始化配置管理器"""
        # 注册默认事件处理器
        validation_handler = ConfigValidationHandler()
        persistence_handler = ConfigPersistenceHandler()
        snapshot_handler = ConfigSnapshotHandler()
        logger_handler = ConfigChangeLogger()

        await self.event_bus.subscribe(ConfigEventType.CONFIG_CHANGED, validation_handler)
        await self.event_bus.subscribe(ConfigEventType.CONFIG_CHANGED, persistence_handler)
        await self.event_bus.subscribe(ConfigEventType.CONFIG_CHANGED, snapshot_handler)
        await self.event_bus.subscribe(ConfigEventType.CONFIG_CHANGED, logger_handler)

        self._handlers.extend([validation_handler, persistence_handler, snapshot_handler, logger_handler])

        logger.info("Event-driven config manager initialized")

    async def get_config(self, key: str, default: Any = None) -> Any:
        return await self.provider.get_config(key, default)

    async def set_config(self, key: str, value: Any, persist: bool = True) -> bool:
        return await self.provider.set_config(key, value, persist)

    async def load_config(self, config: Dict[str, Any], source: str = "loaded"):
        await self.provider.load_config_from_dict(config, source)

    def add_custom_handler(self, handler: IEventHandler):
        """添加自定义事件处理器"""
        self._handlers.append(handler)

    async def shutdown(self):
        """关闭配置管理器"""
        # 清理资源
        pass


# 使用示例
async def setup_event_driven_config():
    """设置事件驱动配置系统"""
    manager = EventDrivenConfigManager()
    await manager.initialize()

    # 加载初始配置
    initial_config = {
        "server.port": 8080,
        "database.url": "sqlite:///app.db",
        "logging.level": "INFO"
    }
    await manager.load_config(initial_config, "initial")

    return manager


# 自定义事件处理器示例
class ConfigChangeNotifier(IEventHandler):
    """配置变更通知器"""

    @property
    def handled_events(self) -> List[ConfigEventType]:
        return [ConfigEventType.CONFIG_CHANGED]

    async def handle(self, event: ConfigEvent) -> None:
        """发送通知"""
        if event.key and event.key.startswith("notification."):
            # 发送通知逻辑
            print(f"Notification triggered by config change: {event.key} = {event.new_value}")


# 解耦的服务注册
class ServiceRegistry:
    """通过事件系统解耦的服务注册表"""

    def __init__(self, event_bus: Optional[EventBus] = None):
        self.event_bus = event_bus or get_event_bus()
        self._services: Dict[str, Any] = {}

    async def register_service(self, name: str, service_config: Dict[str, Any]):
        """注册服务"""
        self._services[name] = service_config

        # 发布服务注册事件
        await self.event_bus.publish(ConfigEvent(
            event_type=ConfigEventType.SERVICE_REGISTERED,
            key=f"services.{name}",
            new_value=service_config,
            source="service_registry"
        ))


# 通过事件系统实现的配置模块间通信
class ConfigModuleCommunicator:
    """配置模块通信器"""

    def __init__(self, module_name: str, event_bus: Optional[EventBus] = None):
        self.module_name = module_name
        self.event_bus = event_bus or get_event_bus()
        self._pending_requests: Dict[str, asyncio.Future] = {}

    async def request_config(self, key: str, target_module: str) -> Any:
        """请求其他模块的配置"""
        correlation_id = f"{self.module_name}-{target_module}-{key}-{id(asyncio.current_task())}"

        # 创建Future等待响应
        future = asyncio.Future()
        self._pending_requests[correlation_id] = future

        # 发布请求事件
        await self.event_bus.publish(ConfigEvent(
            event_type=ConfigEventType.CONFIG_CHANGED,  # 复用现有事件类型
            key=key,
            source=self.module_name,
            correlation_id=correlation_id,
            metadata={"request_type": "get_config", "target_module": target_module}
        ))

        # 等待响应
        try:
            result = await asyncio.wait_for(future, timeout=5.0)
            return result
        except asyncio.TimeoutError:
            raise TimeoutError(f"Config request timeout: {key} from {target_module}")
        finally:
            self._pending_requests.pop(correlation_id, None)


"""
依赖注入容器 - 循环导入的优雅解决方案
通过依赖注入解决循环导入问题，提高代码的可测试性和可维护性
"""

import asyncio
import logging
from dataclasses import dataclass
from enum import Enum
from typing import Dict, Any, Type, TypeVar, Callable, Optional

logger = logging.getLogger(__name__)

T = TypeVar('T')


class DependencyScope(Enum):
    """依赖生命周期范围"""
    SINGLETON = "singleton"        # 单例，整个应用生命周期内只创建一次
    TRANSIENT = "transient"        # 瞬时，每次请求都创建新实例
    SCOPED = "scoped"              # 作用域，在特定范围内复用（如asyncio.Task）


@dataclass
class ServiceDescriptor:
    """服务描述符"""
    interface: Type
    implementation: Optional[Type] = None
    factory: Optional[Callable] = None
    instance: Optional[Any] = None
    scope: DependencyScope = DependencyScope.SINGLETON
    dependencies: Dict[str, str] = None  # 依赖映射：参数名 -> 服务名

    def __post_init__(self):
        if self.dependencies is None:
            self.dependencies = {}


class DIContainer:
    """依赖注入容器"""

    def __init__(self):
        self._services: Dict[str, ServiceDescriptor] = {}
        self._singletons: Dict[str, Any] = {}
        self._scoped: Dict[str, Dict[str, Any]] = {}  # task_id -> instances
        self._lock = asyncio.Lock()

    def register_singleton(self, service_name: str, interface: Type,
                          implementation: Type = None, factory: Callable = None,
                          instance: Any = None):
        """注册单例服务"""
        descriptor = ServiceDescriptor(
            interface=interface,
            implementation=implementation,
            factory=factory,
            instance=instance,
            scope=DependencyScope.SINGLETON
        )
        self._services[service_name] = descriptor

    def register_transient(self, service_name: str, interface: Type,
                          implementation: Type = None, factory: Callable = None):
        """注册瞬时服务"""
        descriptor = ServiceDescriptor(
            interface=interface,
            implementation=implementation,
            factory=factory,
            scope=DependencyScope.TRANSIENT
        )
        self._services[service_name] = descriptor

    def register_scoped(self, service_name: str, interface: Type,
                       implementation: Type = None, factory: Callable = None):
        """注册作用域服务"""
        descriptor = ServiceDescriptor(
            interface=interface,
            implementation=implementation,
            factory=factory,
            scope=DependencyScope.SCOPED
        )
        self._services[service_name] = descriptor

    async def resolve(self, service_name: str) -> Any:
        """解析服务"""
        async with self._lock:
            if service_name not in self._services:
                raise ValueError(f"Service {service_name} not registered")

            descriptor = self._services[service_name]

            # 检查作用域
            if descriptor.scope == DependencyScope.SINGLETON:
                return await self._resolve_singleton(service_name, descriptor)
            elif descriptor.scope == DependencyScope.SCOPED:
                return await self._resolve_scoped(service_name, descriptor)
            else:  # TRANSIENT
                return await self._resolve_transient(descriptor)

    async def _resolve_singleton(self, service_name: str,
                               descriptor: ServiceDescriptor) -> Any:
        """解析单例服务"""
        if service_name in self._singletons:
            return self._singletons[service_name]

        instance = await self._create_instance(descriptor)
        self._singletons[service_name] = instance
        return instance

    async def _resolve_scoped(self, service_name: str,
                            descriptor: ServiceDescriptor) -> Any:
        """解析作用域服务"""
        task_id = id(asyncio.current_task())

        if task_id not in self._scoped:
            self._scoped[task_id] = {}

        if service_name in self._scoped[task_id]:
            return self._scoped[task_id][service_name]

        instance = await self._create_instance(descriptor)
        self._scoped[task_id][service_name] = instance
        return instance

    async def _resolve_transient(self, descriptor: ServiceDescriptor) -> Any:
        """解析瞬时服务"""
        return await self._create_instance(descriptor)

    async def _create_instance(self, descriptor: ServiceDescriptor) -> Any:
        """创建服务实例"""
        # 如果已有实例（单例预注册）
        if descriptor.instance is not None:
            return descriptor.instance

        # 使用工厂方法
        if descriptor.factory:
            return await self._invoke_factory(descriptor.factory)

        # 使用实现类
        if descriptor.implementation:
            return await self._create_implementation(descriptor)

        raise ValueError(f"No way to create instance for {descriptor.interface}")

    async def _invoke_factory(self, factory: Callable) -> Any:
        """调用工厂方法"""
        # 检查是否需要注入依赖
        import inspect
        sig = inspect.signature(factory)

        if not sig.parameters:
            # 无参数工厂
            if asyncio.iscoroutinefunction(factory):
                return await factory()
            else:
                return factory()

        # 需要依赖注入
        kwargs = {}
        for param_name, param in sig.parameters.items():
            if param_name in ['container', 'di_container']:
                kwargs[param_name] = self
            else:
                # 尝试从容器解析依赖
                dependency_name = param_name
                service_desc = self._services.get(dependency_name)
                if service_desc:
                    kwargs[param_name] = await self.resolve(dependency_name)

        if asyncio.iscoroutinefunction(factory):
            return await factory(**kwargs)
        else:
            return factory(**kwargs)

    async def _create_implementation(self, descriptor: ServiceDescriptor) -> Any:
        """创建实现类实例"""
        # 解析依赖
        dependencies = {}
        for dep_param, dep_service in descriptor.dependencies.items():
            dependencies[dep_param] = await self.resolve(dep_service)

        # 创建实例
        instance = descriptor.implementation(**dependencies)
        return instance

    def clear_scoped(self, task_id: int = None):
        """清理作用域实例"""
        if task_id is None:
            task_id = id(asyncio.current_task())
        self._scoped.pop(task_id, None)

    async def dispose(self):
        """释放容器资源"""
        # 清理单例
        for instance in self._singletons.values():
            if hasattr(instance, 'close') and asyncio.iscoroutinefunction(instance.close):
                await instance.close()

        # 清理作用域
        self._scoped.clear()
        self._singletons.clear()


# 全局容器实例
_global_container: Optional[DIContainer] = None


def get_container() -> DIContainer:
    """获取全局依赖注入容器"""
    global _global_container
    if _global_container is None:
        _global_container = DIContainer()
    return _global_container


async def configure_services(container: DIContainer):
    """配置所有服务"""
    # 配置配置服务
    container.register_singleton(
        "config_service",
        interface=object,  # 实际应该是 IConfigService 接口
        factory=lambda container: container.resolve("config_factory")
    )

    # 配置配置工厂
    container.register_singleton(
        "config_factory",
        interface=object,
        factory=create_config_service
    )

    # 配置其他服务...
    logger.info("Dependency injection container configured")


async def create_config_service(container: DIContainer) -> 'ConfigService':
    """创建配置服务的工厂方法"""
    # 延迟导入避免循环依赖
    from mcpstore.core.configuration.config_service import ConfigService

    # 获取依赖的KV存储
    kv_store = await container.resolve("kv_store")

    return ConfigService(kv_store)


# 使用示例
class ExampleService:
    """示例服务，展示依赖注入"""

    def __init__(self, config_service: 'ConfigService', container: DIContainer):
        self.config_service = config_service
        self.container = container

    async def get_config(self, key: str) -> Any:
        return await self.config_service.get_config(key)


async def setup_di_container():
    """设置依赖注入容器"""
    container = get_container()

    # 注册核心服务
    container.register_singleton(
        "kv_store",
        interface=object,
        factory=lambda: MockAsyncKeyValue()  # 或者真实的KV存储
    )

    await configure_services(container)

    return container


"""
配置系统接口定义
通过接口抽象化解决循环导入，实现依赖反转
"""

from abc import ABC, abstractmethod
from dataclasses import dataclass
from enum import Enum
from typing import Dict, Any, Optional, List, Set


class ConfigSource(Enum):
    """配置源类型"""
    DEFAULT = "default"
    TOML_FILE = "toml_file"
    KV_STORAGE = "kv_storage"
    ENVIRONMENT = "environment"


@dataclass
class ConfigItem:
    """配置项"""
    key: str
    value: Any
    source: ConfigSource
    metadata: Optional[Dict[str, Any]] = None


@dataclass
class ConfigValidationRule:
    """配置验证规则"""
    required: bool = False
    min_value: Optional[float] = None
    max_value: Optional[float] = None
    allowed_values: Optional[List[Any]] = None
    data_type: Optional[type] = None


class IConfigProvider(ABC):
    """配置提供者接口"""

    @abstractmethod
    async def get_config(self, key: str, default: Any = None) -> Any:
        """获取配置值"""
        pass

    @abstractmethod
    async def set_config(self, key: str, value: Any, persist: bool = True) -> bool:
        """设置配置值"""
        pass

    @abstractmethod
    async def get_config_with_source(self, key: str) -> Optional[ConfigItem]:
        """获取配置项和源信息"""
        pass

    @abstractmethod
    async def list_configs(self, pattern: str = "*") -> List[ConfigItem]:
        """列出配置项"""
        pass

    @abstractmethod
    async def validate_config(self, key: str, value: Any) -> tuple[bool, str]:
        """验证配置值"""
        pass


class IConfigValidator(ABC):
    """配置验证器接口"""

    @abstractmethod
    def get_validation_rule(self, key: str) -> Optional[ConfigValidationRule]:
        """获取验证规则"""
        pass

    @abstractmethod
    def validate_value(self, key: str, value: Any) -> tuple[bool, str]:
        """验证值"""
        pass

    @abstractmethod
    def get_supported_keys(self) -> Set[str]:
        """获取支持的配置键"""
        pass


class IConfigPersistence(ABC):
    """配置持久化接口"""

    @abstractmethod
    async def load_config(self) -> Dict[str, Any]:
        """加载配置"""
        pass

    @abstractmethod
    async def save_config(self, config: Dict[str, Any]) -> bool:
        """保存配置"""
        pass

    @abstractmethod
    async def backup_config(self) -> bool:
        """备份配置"""
        pass

    @abstractmethod
    async def restore_config(self) -> bool:
        """恢复配置"""
        pass


class IConfigSnapshot(ABC):
    """配置快照接口"""

    @abstractmethod
    async def create_snapshot(self) -> Dict[str, Any]:
        """创建快照"""
        pass

    @abstractmethod
    async def get_snapshot(self, snapshot_id: str) -> Dict[str, Any]:
        """获取快照"""
        pass

    @abstractmethod
    async def list_snapshots(self) -> List[Dict[str, Any]]:
        """列出快照"""
        pass


class IConfigMetadata(ABC):
    """配置元数据接口"""

    @abstractmethod
    def get_metadata(self, key: str) -> Optional[Dict[str, Any]]:
        """获取配置元数据"""
        pass

    @abstractmethod
    def get_all_metadata(self) -> Dict[str, Dict[str, Any]]:
        """获取所有元数据"""
        pass

    @abstractmethod
    def get_dynamic_keys(self) -> Set[str]:
        """获取动态配置键"""
        pass


class IConfigFactory(ABC):
    """配置工厂接口"""

    @abstractmethod
    async def create_config_provider(self) -> IConfigProvider:
        """创建配置提供者"""
        pass

    @abstractmethod
    async def create_validator(self) -> IConfigValidator:
        """创建验证器"""
        pass

    @abstractmethod
    async def create_persistence(self) -> IConfigPersistence:
        """创建持久化"""
        pass


class IConfigService(IConfigProvider, IConfigMetadata):
    """配置服务组合接口"""

    @abstractmethod
    async def get_all_configs(self) -> Dict[str, Any]:
        """获取所有配置"""
        pass

    @abstractmethod
    async def update_configs(self, updates: Dict[str, Any]) -> Dict[str, bool]:
        """批量更新配置"""
        pass

    @abstractmethod
    async def reload_config(self) -> bool:
        """重新加载配置"""
        pass

    @abstractmethod
    async def export_config(self, format: str = "json") -> str:
        """导出配置"""
        pass


# 配置类型接口
class IContentUpdateConfig(ABC):
    """内容更新配置接口"""

    @abstractmethod
    def get_tools_update_interval(self) -> float:
        pass

    @abstractmethod
    def is_auto_update_enabled(self) -> bool:
        pass


class IServiceLifecycleConfig(ABC):
    """服务生命周期配置接口"""

    @abstractmethod
    def get_warning_threshold(self) -> int:
        pass

    @abstractmethod
    def get_reconnect_attempts(self) -> int:
        pass


class ICacheConfig(ABC):
    """缓存配置接口"""

    @abstractmethod
    def get_cache_type(self) -> str:
        pass

    @abstractmethod
    def get_cache_settings(self) -> Dict[str, Any]:
        pass


class IServerConfig(ABC):
    """服务器配置接口"""

    @abstractmethod
    def get_server_settings(self) -> Dict[str, Any]:
        pass

    @abstractmethod
    def get_port(self) -> int:
        pass


# 抽象基类实现
class BaseConfigProvider(IConfigProvider):
    """配置提供者基类"""

    def __init__(self):
        self._validators: List[IConfigValidator] = []

    def add_validator(self, validator: IConfigValidator):
        self._validators.append(validator)

    async def validate_config(self, key: str, value: Any) -> tuple[bool, str]:
        for validator in self._validators:
            is_valid, message = validator.validate_value(key, value)
            if not is_valid:
                return False, message
        return True, ""


class BaseConfigValidator(IConfigValidator):
    """配置验证器基类"""

    def __init__(self):
        self._rules: Dict[str, ConfigValidationRule] = {}

    def add_rule(self, key: str, rule: ConfigValidationRule):
        self._rules[key] = rule

    def get_validation_rule(self, key: str) -> Optional[ConfigValidationRule]:
        return self._rules.get(key)

    def validate_value(self, key: str, value: Any) -> tuple[bool, str]:
        rule = self.get_validation_rule(key)
        if not rule:
            return True, ""

        # 类型检查
        if rule.data_type and not isinstance(value, rule.data_type):
            return False, f"Expected type {rule.data_type.__name__}, got {type(value).__name__}"

        # 必填检查
        if rule.required and value is None:
            return False, "This configuration is required"

        # 范围检查
        if rule.min_value is not None and value < rule.min_value:
            return False, f"Value must be >= {rule.min_value}"

        if rule.max_value is not None and value > rule.max_value:
            return False, f"Value must be <= {rule.max_value}"

        # 枚举值检查
        if rule.allowed_values and value not in rule.allowed_values:
            return False, f"Value must be one of {rule.allowed_values}"

        return True, ""


# 工厂接口实现
class ConfigFactoryBase(IConfigFactory):
    """配置工厂基类"""

    def __init__(self, container=None):
        self.container = container

    async def create_config_service(self) -> IConfigService:
        provider = await self.create_config_provider()
        validator = await self.create_validator()

        # 组合服务
        service = CompositeConfigService(provider, validator)
        return service

    async def create_validator(self) -> IConfigValidator:
        validator = BaseConfigValidator()
        # 添加验证规则
        self._configure_validator(validator)
        return validator

    def _configure_validator(self, validator: BaseConfigValidator):
        """配置验证规则 - 子类实现"""
        pass


class CompositeConfigService(IConfigService):
    """组合配置服务"""

    def __init__(self, provider: IConfigProvider, validator: IConfigValidator):
        self._provider = provider
        self._validator = validator

    async def get_config(self, key: str, default: Any = None) -> Any:
        return await self._provider.get_config(key, default)

    async def set_config(self, key: str, value: Any, persist: bool = True) -> bool:
        # 验证
        is_valid, message = await self._validator.validate_value(key, value)
        if not is_valid:
            raise ValueError(f"Invalid config {key}: {message}")

        return await self._provider.set_config(key, value, persist)

    # 委托其他方法到提供者
    async def get_config_with_source(self, key: str) -> Optional[ConfigItem]:
        return await self._provider.get_config_with_source(key)

    async def list_configs(self, pattern: str = "*") -> List[ConfigItem]:
        return await self._provider.list_configs(pattern)

    async def validate_config(self, key: str, value: Any) -> tuple[bool, str]:
        return await self._validator.validate_value(key, value)

    async def get_all_configs(self) -> Dict[str, Any]:
        configs = await self.list_configs()
        return {item.key: item.value for item in configs}

    async def update_configs(self, updates: Dict[str, Any]) -> Dict[str, bool]:
        results = {}
        for key, value in updates.items():
            try:
                results[key] = await self.set_config(key, value)
            except Exception:
                results[key] = False
        return results

    async def reload_config(self) -> bool:
        # 实现重新加载逻辑
        return True

    async def export_config(self, format: str = "json") -> str:
        # 实现导出逻辑
        configs = await self.get_all_configs()
        import json
        return json.dumps(configs, indent=2)

    def get_metadata(self, key: str) -> Optional[Dict[str, Any]]:
        return self._validator.get_validation_rule(key).__dict__ if self._validator.get_validation_rule(key) else None

    def get_all_metadata(self) -> Dict[str, Dict[str, Any]]:
        return {key: rule.__dict__ for key, rule in self._validator._rules.items()}

    def get_dynamic_keys(self) -> Set[str]:
        return set(self._validator._rules.keys())